@@ -13,43 +13,47 @@
 #   under the License.
 #
 
-"""Modify Cliff's CommandManager"""
+"""Modify cliff.CommandManager"""
 
-import logging
 import pkg_resources
 
 import cliff.commandmanager
 
 
-LOG = logging.getLogger(__name__)
-
-
 class CommandManager(cliff.commandmanager.CommandManager):
-    """Alters Cliff's default CommandManager behaviour to load additional
-       command groups after initialization.
+    """Add additional functionality to cliff.CommandManager
+
+    Load additional command groups after initialization
+    Add _command_group() methods
     """
+
     def __init__(self, namespace, convert_underscores=True):
         self.group_list = []
         super(CommandManager, self).__init__(namespace, convert_underscores)
 
-    def _load_commands(self, group=None):
-        if not group:
-            group = self.namespace
-        self.group_list.append(group)
-        for ep in pkg_resources.iter_entry_points(group):
-            cmd_name = (
-                ep.name.replace('_', ' ')
-                if self.convert_underscores
-                else ep.name
-            )
-            self.commands[cmd_name] = ep
-        return
+    def load_commands(self, namespace):
+        self.group_list.append(namespace)
+        return super(CommandManager, self).load_commands(namespace)
 
     def add_command_group(self, group=None):
         """Adds another group of command entrypoints"""
         if group:
-            self._load_commands(group)
+            self.load_commands(group)
 
     def get_command_groups(self):
         """Returns a list of the loaded command groups"""
         return self.group_list
+
+    def get_command_names(self, group=None):
+        """Returns a list of commands loaded for the specified group"""
+        group_list = []
+        if group is not None:
+            for ep in pkg_resources.iter_entry_points(group):
+                cmd_name = (
+                    ep.name.replace('_', ' ')
+                    if self.convert_underscores
+                    else ep.name
+                )
+                group_list.append(cmd_name)
+            return group_list
+        return self.commands.keys()
