@@ -13,19 +13,13 @@
 
 """Timing Implementation"""
 
-import logging
-
-from cliff import lister
+from openstackclient.common import command
 
 
-class Timing(lister.Lister):
+class Timing(command.Lister):
     """Show timing data"""
 
-    log = logging.getLogger(__name__ + '.Timing')
-
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)' % parsed_args)
-
         column_headers = (
             'URL',
             'Seconds',
@@ -36,7 +30,8 @@ class Timing(lister.Lister):
         for url, td in self.app.timing_data:
             # NOTE(dtroyer): Take the long way here because total_seconds()
             #                was added in py27.
-            sec = (td.microseconds + (td.seconds + td.days*86400) * 1e6) / 1e6
+            sec = (td.microseconds + (td.seconds + td.days *
+                                      86400) * 1e6) / 1e6
             total += sec
             results.append((url, sec))
         results.append(('Total', total))
