@@ -16,39 +16,31 @@
 
 """Compute v2 Aggregate action implementations"""
 
-import logging
 import six
 
-from cliff import command
-from cliff import lister
-from cliff import show
-
+from openstackclient.common import command
 from openstackclient.common import parseractions
 from openstackclient.common import utils
 
 
-class AddAggregateHost(show.ShowOne):
+class AddAggregateHost(command.ShowOne):
     """Add host to aggregate"""
 
-    log = logging.getLogger(__name__ + '.AddAggregateHost')
-
     def get_parser(self, prog_name):
         parser = super(AddAggregateHost, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Name or ID of aggregate',
+            help='Aggregate (name or ID)',
         )
         parser.add_argument(
             'host',
             metavar='<host>',
-            help='Host to add to aggregate',
+            help='Host to add to <aggregate>',
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
-
         compute_client = self.app.client_manager.compute
 
         aggregate = utils.find_resource(
@@ -62,11 +54,9 @@ class AddAggregateHost(show.ShowOne):
         return zip(*sorted(six.iteritems(info)))
 
 
-class CreateAggregate(show.ShowOne):
+class CreateAggregate(command.ShowOne):
     """Create a new aggregate"""
 
-    log = logging.getLogger(__name__ + ".CreateAggregate")
-
     def get_parser(self, prog_name):
         parser = super(CreateAggregate, self).get_parser(prog_name)
         parser.add_argument(
@@ -89,8 +79,6 @@ class CreateAggregate(show.ShowOne):
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
-
         compute_client = self.app.client_manager.compute
 
         info = {}
@@ -112,19 +100,16 @@ class CreateAggregate(show.ShowOne):
 class DeleteAggregate(command.Command):
     """Delete an existing aggregate"""
 
-    log = logging.getLogger(__name__ + '.DeleteAggregate')
-
     def get_parser(self, prog_name):
         parser = super(DeleteAggregate, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Name or ID of aggregate to delete',
+            help='Aggregate to delete (name or ID)',
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         compute_client = self.app.client_manager.compute
         data = utils.find_resource(
@@ -132,14 +117,11 @@ class DeleteAggregate(command.Command):
             parsed_args.aggregate,
         )
         compute_client.aggregates.delete(data.id)
-        return
 
 
-class ListAggregate(lister.Lister):
+class ListAggregate(command.Lister):
     """List all aggregates"""
 
-    log = logging.getLogger(__name__ + ".ListAggregate")
-
     def get_parser(self, prog_name):
         parser = super(ListAggregate, self).get_parser(prog_name)
         parser.add_argument(
@@ -150,8 +132,6 @@ class ListAggregate(lister.Lister):
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
-
         compute_client = self.app.client_manager.compute
 
         data = compute_client.aggregates.list()
@@ -187,28 +167,24 @@ class ListAggregate(lister.Lister):
                 ) for s in data))
 
 
-class RemoveAggregateHost(show.ShowOne):
+class RemoveAggregateHost(command.ShowOne):
     """Remove host from aggregate"""
 
-    log = logging.getLogger(__name__ + '.RemoveAggregateHost')
-
     def get_parser(self, prog_name):
         parser = super(RemoveAggregateHost, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Name or ID of aggregate',
+            help='Aggregate (name or ID)',
         )
         parser.add_argument(
             'host',
             metavar='<host>',
-            help='Host to remove from aggregate',
+            help='Host to remove from <aggregate>',
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
-
         compute_client = self.app.client_manager.compute
 
         aggregate = utils.find_resource(
@@ -225,39 +201,36 @@ class RemoveAggregateHost(show.ShowOne):
         return zip(*sorted(six.iteritems(info)))
 
 
-class SetAggregate(show.ShowOne):
+class SetAggregate(command.Command):
     """Set aggregate properties"""
 
-    log = logging.getLogger(__name__ + '.SetAggregate')
-
     def get_parser(self, prog_name):
         parser = super(SetAggregate, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Name or ID of aggregate to display',
+            help='Aggregate to modify (name or ID)',
         )
         parser.add_argument(
             '--name',
-            metavar='<new-name>',
-            help='New aggregate name',
+            metavar='<name>',
+            help='Set aggregate name',
         )
         parser.add_argument(
             "--zone",
             metavar="<availability-zone>",
-            help="Availability zone name",
+            help="Set availability zone name",
         )
         parser.add_argument(
             "--property",
             metavar="<key=value>",
             action=parseractions.KeyValueAction,
-            help='Property to add/change for this aggregate '
+            help='Property to set on <aggregate> '
                  '(repeat option to set multiple properties)',
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         compute_client = self.app.client_manager.compute
         aggregate = utils.find_resource(
@@ -265,46 +238,37 @@ class SetAggregate(show.ShowOne):
             parsed_args.aggregate,
         )
 
-        info = {}
         kwargs = {}
         if parsed_args.name:
             kwargs['name'] = parsed_args.name
         if parsed_args.zone:
             kwargs['availability_zone'] = parsed_args.zone
         if kwargs:
-            info.update(compute_client.aggregates.update(
+            compute_client.aggregates.update(
                 aggregate,
                 kwargs
-            )._info)
+            )
 
         if parsed_args.property:
-            info.update(compute_client.aggregates.set_metadata(
+            compute_client.aggregates.set_metadata(
                 aggregate,
-                parsed_args.property,
-            )._info)
-
-        if info:
-            return zip(*sorted(six.iteritems(info)))
-        else:
-            return ({}, {})
+                parsed_args.property
+            )
 
 
-class ShowAggregate(show.ShowOne):
-    """Show a specific aggregate"""
-
-    log = logging.getLogger(__name__ + '.ShowAggregate')
+class ShowAggregate(command.ShowOne):
+    """Display aggregate details"""
 
     def get_parser(self, prog_name):
         parser = super(ShowAggregate, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Name or ID of aggregate to display',
+            help='Aggregate to display (name or ID)',
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         compute_client = self.app.client_manager.compute
         data = utils.find_resource(
@@ -314,8 +278,14 @@ class ShowAggregate(show.ShowOne):
         # Remove availability_zone from metadata because Nova doesn't
         if 'availability_zone' in data.metadata:
             data.metadata.pop('availability_zone')
-        # Map 'metadata' column to 'properties'
-        data._info.update({'properties': data._info.pop('metadata')})
+
+        # Special mapping for columns to make the output easier to read:
+        # 'metadata' --> 'properties'
+        data._info.update(
+            {
+                'properties': utils.format_dict(data._info.pop('metadata')),
+            },
+        )
 
         info = {}
         info.update(data._info)
