@@ -15,20 +15,14 @@
 
 """Floating IP Pool action implementations"""
 
-import logging
-
-from cliff import lister
-
+from openstackclient.common import command
 from openstackclient.common import utils
 
 
-class ListFloatingIPPool(lister.Lister):
-    """List floating-ip-pool command"""
-
-    log = logging.getLogger(__name__ + '.ListFloatingIPPool')
+class ListFloatingIPPool(command.Lister):
+    """List pools of floating IP addresses"""
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         compute_client = self.app.client_manager.compute
 
         columns = ('Name',)
