@@ -15,56 +15,57 @@
 
 """Identity v2 Project action implementations"""
 
-import logging
 import six
 
-from cliff import command
-from cliff import lister
-from cliff import show
+from keystoneauth1 import exceptions as ks_exc
 
+from openstackclient.common import command
 from openstackclient.common import parseractions
 from openstackclient.common import utils
+from openstackclient.i18n import _  # noqa
 
 
-class CreateProject(show.ShowOne):
+class CreateProject(command.ShowOne):
     """Create new project"""
 
-    log = logging.getLogger(__name__ + '.CreateProject')
-
     def get_parser(self, prog_name):
         parser = super(CreateProject, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<project-name>',
-            help='New project name',
+            help=_('New project name'),
         )
         parser.add_argument(
             '--description',
-            metavar='<project-description>',
-            help='New project description',
+            metavar='<description>',
+            help=_('Project description'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
             '--enable',
             action='store_true',
-            help='Enable project (default)',
+            help=_('Enable project (default)'),
         )
         enable_group.add_argument(
             '--disable',
             action='store_true',
-            help='Disable project',
+            help=_('Disable project'),
         )
         parser.add_argument(
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help='Property to add for this project '
-                 '(repeat option to set multiple properties)',
+            help=_('Add a property to <name> '
+                   '(repeat option to set multiple properties)'),
+        )
+        parser.add_argument(
+            '--or-show',
+            action='store_true',
+            help=_('Return existing project'),
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
         enabled = True
@@ -74,62 +75,67 @@ class CreateProject(show.ShowOne):
         if parsed_args.property:
             kwargs = parsed_args.property.copy()
 
-        project = identity_client.tenants.create(
-            parsed_args.name,
-            description=parsed_args.description,
-            enabled=enabled,
-            **kwargs
-        )
+        try:
+            project = identity_client.tenants.create(
+                parsed_args.name,
+                description=parsed_args.description,
+                enabled=enabled,
+                **kwargs
+            )
+        except ks_exc.Conflict as e:
+            if parsed_args.or_show:
+                project = utils.find_resource(
+                    identity_client.tenants,
+                    parsed_args.name,
+                )
+                self.log.info('Returning existing project %s', project.name)
+            else:
+                raise e
 
-        info = {}
-        info.update(project._info)
-        return zip(*sorted(six.iteritems(info)))
+        # TODO(stevemar): Remove the line below when we support multitenancy
+        project._info.pop('parent_id', None)
+        return zip(*sorted(six.iteritems(project._info)))
 
 
 class DeleteProject(command.Command):
-    """Delete project"""
-
-    log = logging.getLogger(__name__ + '.DeleteProject')
+    """Delete project(s)"""
 
     def get_parser(self, prog_name):
         parser = super(DeleteProject, self).get_parser(prog_name)
         parser.add_argument(
-            'project',
+            'projects',
             metavar='<project>',
-            help='Project to delete (name or ID)',
+            nargs="+",
+            help=_('Project(s) to delete (name or ID)'),
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
-        project = utils.find_resource(
-            identity_client.tenants,
-            parsed_args.project,
-        )
-
-        identity_client.tenants.delete(project.id)
+        for project in parsed_args.projects:
+            project_obj = utils.find_resource(
+                identity_client.tenants,
+                project,
+            )
+            identity_client.tenants.delete(project_obj.id)
         return
 
 
-class ListProject(lister.Lister):
+class ListProject(command.Lister):
     """List projects"""
 
-    log = logging.getLogger(__name__ + '.ListProject')
-
     def get_parser(self, prog_name):
         parser = super(ListProject, self).get_parser(prog_name)
         parser.add_argument(
             '--long',
             action='store_true',
             default=False,
-            help='List additional fields in output',
+            help=_('List additional fields in output'),
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         if parsed_args.long:
             columns = ('ID', 'Name', 'Description', 'Enabled')
         else:
@@ -145,47 +151,44 @@ class ListProject(lister.Lister):
 class SetProject(command.Command):
     """Set project properties"""
 
-    log = logging.getLogger(__name__ + '.SetProject')
-
     def get_parser(self, prog_name):
         parser = super(SetProject, self).get_parser(prog_name)
         parser.add_argument(
             'project',
             metavar='<project>',
-            help='Project to change (name or ID)',
+            help=_('Project to modify (name or ID)'),
         )
         parser.add_argument(
             '--name',
-            metavar='<new-project-name>',
-            help='New project name',
+            metavar='<name>',
+            help=_('Set project name'),
         )
         parser.add_argument(
             '--description',
-            metavar='<project-description>',
-            help='New project description',
+            metavar='<description>',
+            help=_('Set project description'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
             '--enable',
             action='store_true',
-            help='Enable project',
+            help=_('Enable project'),
         )
         enable_group.add_argument(
             '--disable',
             action='store_true',
-            help='Disable project',
+            help=_('Disable project'),
         )
         parser.add_argument(
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help='Property to add for this project '
-                 '(repeat option to set multiple properties)',
+            help=_('Set a project property '
+                   '(repeat option to set multiple properties)'),
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
         if (not parsed_args.name
@@ -222,27 +225,96 @@ class SetProject(command.Command):
         return
 
 
-class ShowProject(show.ShowOne):
-    """Show project details"""
-
-    log = logging.getLogger(__name__ + '.ShowProject')
+class ShowProject(command.ShowOne):
+    """Display project details"""
 
     def get_parser(self, prog_name):
         parser = super(ShowProject, self).get_parser(prog_name)
         parser.add_argument(
             'project',
             metavar='<project>',
-            help='Project to display (name or ID)')
+            help=_('Project to display (name or ID)'))
+        return parser
+
+    def take_action(self, parsed_args):
+        identity_client = self.app.client_manager.identity
+
+        info = {}
+        try:
+            project = utils.find_resource(
+                identity_client.tenants,
+                parsed_args.project,
+            )
+            info.update(project._info)
+        except ks_exc.Forbidden as e:
+            auth_ref = self.app.client_manager.auth_ref
+            if (
+                parsed_args.project == auth_ref.project_id or
+                parsed_args.project == auth_ref.project_name
+            ):
+                # Ask for currently auth'ed project so return it
+                info = {
+                    'id': auth_ref.project_id,
+                    'name': auth_ref.project_name,
+                    # True because we don't get this far if it is disabled
+                    'enabled': True,
+                }
+            else:
+                raise e
+
+        # TODO(stevemar): Remove the line below when we support multitenancy
+        info.pop('parent_id', None)
+
+        # NOTE(stevemar): Property handling isn't really supported in Keystone
+        # and needs a lot of extra handling. Let's reserve the properties that
+        # the API has and handle the extra top level properties.
+        reserved = ('name', 'id', 'enabled', 'description')
+        properties = {}
+        for k, v in info.items():
+            if k not in reserved:
+                # If a key is not in `reserved` it's a property, pop it
+                info.pop(k)
+                # If a property has been "unset" it's `None`, so don't show it
+                if v is not None:
+                    properties[k] = v
+
+        info['properties'] = utils.format_dict(properties)
+        return zip(*sorted(six.iteritems(info)))
+
+
+class UnsetProject(command.Command):
+    """Unset project properties"""
+
+    def get_parser(self, prog_name):
+        parser = super(UnsetProject, self).get_parser(prog_name)
+        parser.add_argument(
+            'project',
+            metavar='<project>',
+            help=_('Project to modify (name or ID)'),
+        )
+        parser.add_argument(
+            '--property',
+            metavar='<key>',
+            action='append',
+            default=[],
+            help=_('Unset a project property '
+                   '(repeat option to unset multiple properties)'),
+            required=True,
+        )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         project = utils.find_resource(
             identity_client.tenants,
             parsed_args.project,
         )
-
-        info = {}
-        info.update(project._info)
-        return zip(*sorted(six.iteritems(info)))
+        if not parsed_args.property:
+            self.app.log.error("No changes requested\n")
+        else:
+            kwargs = project._info
+            for key in parsed_args.property:
+                if key in kwargs:
+                    kwargs[key] = None
+            identity_client.tenants.update(project.id, **kwargs)
+        return
