@@ -13,21 +13,15 @@
 
 """Identity v3 IdentityProvider action implementations"""
 
-import logging
 import six
 
-from cliff import command
-from cliff import lister
-from cliff import show
-
+from openstackclient.common import command
 from openstackclient.common import utils
 
 
-class CreateIdentityProvider(show.ShowOne):
+class CreateIdentityProvider(command.ShowOne):
     """Create new identity provider"""
 
-    log = logging.getLogger(__name__ + '.CreateIdentityProvider')
-
     def get_parser(self, prog_name):
         parser = super(CreateIdentityProvider, self).get_parser(prog_name)
         parser.add_argument(
@@ -70,7 +64,6 @@ class CreateIdentityProvider(show.ShowOne):
         )
         return parser
 
-    @utils.log_method(log)
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
         if parsed_args.remote_id_file:
@@ -88,14 +81,14 @@ class CreateIdentityProvider(show.ShowOne):
             enabled=parsed_args.enabled)
 
         idp._info.pop('links', None)
+        remote_ids = utils.format_list(idp._info.pop('remote_ids', []))
+        idp._info['remote_ids'] = remote_ids
         return zip(*sorted(six.iteritems(idp._info)))
 
 
 class DeleteIdentityProvider(command.Command):
     """Delete identity provider"""
 
-    log = logging.getLogger(__name__ + '.DeleteIdentityProvider')
-
     def get_parser(self, prog_name):
         parser = super(DeleteIdentityProvider, self).get_parser(prog_name)
         parser.add_argument(
@@ -105,20 +98,15 @@ class DeleteIdentityProvider(command.Command):
         )
         return parser
 
-    @utils.log_method(log)
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
         identity_client.federation.identity_providers.delete(
             parsed_args.identity_provider)
-        return
 
 
-class ListIdentityProvider(lister.Lister):
+class ListIdentityProvider(command.Lister):
     """List identity providers"""
 
-    log = logging.getLogger(__name__ + '.ListIdentityProvider')
-
-    @utils.log_method(log)
     def take_action(self, parsed_args):
         columns = ('ID', 'Enabled', 'Description')
         identity_client = self.app.client_manager.identity
@@ -133,8 +121,6 @@ class ListIdentityProvider(lister.Lister):
 class SetIdentityProvider(command.Command):
     """Set identity provider properties"""
 
-    log = logging.getLogger(__name__ + '.SetIdentityProvider')
-
     def get_parser(self, prog_name):
         parser = super(SetIdentityProvider, self).get_parser(prog_name)
         parser.add_argument(
@@ -142,6 +128,11 @@ class SetIdentityProvider(command.Command):
             metavar='<identity-provider>',
             help='Identity provider to modify',
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help='Set identity provider description',
+        )
         identity_remote_id_provider = parser.add_mutually_exclusive_group()
         identity_remote_id_provider.add_argument(
             '--remote-id',
@@ -169,13 +160,14 @@ class SetIdentityProvider(command.Command):
         )
         return parser
 
-    @utils.log_method(log)
     def take_action(self, parsed_args):
         federation_client = self.app.client_manager.identity.federation
 
         # Basic argument checking
-        if (not parsed_args.enable and not parsed_args.disable and not
-                parsed_args.remote_id and not parsed_args.remote_id_file):
+        if (not parsed_args.enable and not parsed_args.disable and
+                not parsed_args.remote_id and
+                not parsed_args.remote_id_file and
+                not parsed_args.description):
             self.log.error('No changes requested')
             return (None, None)
 
@@ -190,6 +182,8 @@ class SetIdentityProvider(command.Command):
 
         # Setup keyword args for the client
         kwargs = {}
+        if parsed_args.description:
+            kwargs['description'] = parsed_args.description
         if parsed_args.enable:
             kwargs['enabled'] = True
         if parsed_args.disable:
@@ -204,11 +198,9 @@ class SetIdentityProvider(command.Command):
         return zip(*sorted(six.iteritems(identity_provider._info)))
 
 
-class ShowIdentityProvider(show.ShowOne):
+class ShowIdentityProvider(command.ShowOne):
     """Display identity provider details"""
 
-    log = logging.getLogger(__name__ + '.ShowIdentityProvider')
-
     def get_parser(self, prog_name):
         parser = super(ShowIdentityProvider, self).get_parser(prog_name)
         parser.add_argument(
@@ -218,12 +210,13 @@ class ShowIdentityProvider(show.ShowOne):
         )
         return parser
 
-    @utils.log_method(log)
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
-        identity_provider = utils.find_resource(
+        idp = utils.find_resource(
             identity_client.federation.identity_providers,
             parsed_args.identity_provider)
 
-        identity_provider._info.pop('links', None)
-        return zip(*sorted(six.iteritems(identity_provider._info)))
+        idp._info.pop('links', None)
+        remote_ids = utils.format_list(idp._info.pop('remote_ids', []))
+        idp._info['remote_ids'] = remote_ids
+        return zip(*sorted(six.iteritems(idp._info)))
