@@ -15,7 +15,8 @@
 
 """Image V1 Action Implementations"""
 
-import logging
+import argparse
+import io
 import os
 import six
 import sys
@@ -25,30 +26,41 @@ if os.name == "nt":
 else:
     msvcrt = None
 
-from cliff import command
-from cliff import lister
-from cliff import show
-
 from glanceclient.common import utils as gc_utils
-from openstackclient.common import exceptions
+from openstackclient.api import utils as api_utils
+from openstackclient.common import command
 from openstackclient.common import parseractions
 from openstackclient.common import utils
+from openstackclient.i18n import _  # noqa
 
 
 DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
 
 
-class CreateImage(show.ShowOne):
-    """Create/upload an image"""
+def _format_visibility(data):
+    """Return a formatted visibility string
 
-    log = logging.getLogger(__name__ + ".CreateImage")
+    :param data:
+        The server's visibility (is_public) status value: True, False
+    :rtype:
+        A string formatted to public/private
+    """
+
+    if data:
+        return 'public'
+    else:
+        return 'private'
+
+
+class CreateImage(command.ShowOne):
+    """Create/upload an image"""
 
     def get_parser(self, prog_name):
         parser = super(CreateImage, self).get_parser(prog_name)
         parser.add_argument(
             "name",
-            metavar="<name>",
+            metavar="<image-name>",
             help="New image name",
         )
         parser.add_argument(
@@ -75,11 +87,6 @@ class CreateImage(show.ShowOne):
             help="Image disk format "
                  "(default: %s)" % DEFAULT_DISK_FORMAT,
         )
-        parser.add_argument(
-            "--owner",
-            metavar="<project>",
-            help="Image owner project name or ID",
-        )
         parser.add_argument(
             "--size",
             metavar="<size>",
@@ -158,21 +165,40 @@ class CreateImage(show.ShowOne):
             dest="properties",
             metavar="<key=value>",
             action=parseractions.KeyValueAction,
-            help="Set an image property "
+            help="Set a property on this image "
                  "(repeat option to set multiple properties)",
         )
+        # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
+        #                2.x release.  Do not remove before Jan 2017
+        #                and a 3.x release.
+        project_group = parser.add_mutually_exclusive_group()
+        project_group.add_argument(
+            "--project",
+            metavar="<project>",
+            help="Set an alternate project on this image (name or ID)",
+        )
+        project_group.add_argument(
+            "--owner",
+            metavar="<project>",
+            help=argparse.SUPPRESS,
+        )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
         image_client = self.app.client_manager.image
 
+        if getattr(parsed_args, 'owner', None) is not None:
+            self.log.warning(_(
+                'The --owner option is deprecated, '
+                'please use --project instead.'
+            ))
+
         # Build an attribute dict from the parsed args, only include
         # attributes that were actually set on the command line
         kwargs = {}
         copy_attrs = ('name', 'id', 'store', 'container_format',
                       'disk_format', 'owner', 'size', 'min_disk', 'min_ram',
-                      'localtion', 'copy_from', 'volume', 'force',
+                      'location', 'copy_from', 'volume', 'force',
                       'checksum', 'properties')
         for attr in copy_attrs:
             if attr in parsed_args:
@@ -181,6 +207,12 @@ class CreateImage(show.ShowOne):
                     # Only include a value in kwargs for attributes that are
                     # actually present on the command line
                     kwargs[attr] = val
+
+        # Special case project option back to API attribute name 'owner'
+        val = getattr(parsed_args, 'project', None)
+        if val:
+            kwargs['owner'] = val
+
         # Handle exclusive booleans with care
         # Avoid including attributes in kwargs if an option is not
         # present on the command line.  These exclusive booleans are not
@@ -196,6 +228,8 @@ class CreateImage(show.ShowOne):
         if parsed_args.private:
             kwargs['is_public'] = False
 
+        info = {}
+
         if not parsed_args.location and not parsed_args.copy_from:
             if parsed_args.volume:
                 volume_client = self.app.client_manager.volume
@@ -214,10 +248,9 @@ class CreateImage(show.ShowOne):
             elif parsed_args.file:
                 # Send an open file handle to glanceclient so it will
                 # do a chunked transfer
-                kwargs["data"] = open(parsed_args.file, "rb")
+                kwargs["data"] = io.open(parsed_args.file, "rb")
             else:
                 # Read file from stdin
-                kwargs["data"] = None
                 if sys.stdin.isatty() is not True:
                     if msvcrt:
                         msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
@@ -225,111 +258,200 @@ class CreateImage(show.ShowOne):
                     # do a chunked transfer
                     kwargs["data"] = sys.stdin
 
-        try:
-            image = utils.find_resource(
-                image_client.images,
-                parsed_args.name,
-            )
-
-            # Preserve previous properties if any are being set now
-            if image.properties:
-                if parsed_args.properties:
-                    image.properties.update(kwargs['properties'])
-                kwargs['properties'] = image.properties
-
-        except exceptions.CommandError:
-            if not parsed_args.volume:
-                # This is normal for a create or reserve (create w/o an image)
-                # But skip for create from volume
+        if not parsed_args.volume:
+            # Wrap the call to catch exceptions in order to close files
+            try:
                 image = image_client.images.create(**kwargs)
-        else:
-            # Update an existing reservation
+            finally:
+                # Clean up open files - make sure data isn't a string
+                if ('data' in kwargs and hasattr(kwargs['data'], 'close') and
+                        kwargs['data'] != sys.stdin):
+                    kwargs['data'].close()
 
-            # If an image is specified via --file, --location or
-            # --copy-from let the API handle it
-            image = image_client.images.update(image.id, **kwargs)
-
-        info = {}
-        info.update(image._info)
+            info.update(image._info)
+            info['properties'] = utils.format_dict(info.get('properties', {}))
         return zip(*sorted(six.iteritems(info)))
 
 
 class DeleteImage(command.Command):
-    """Delete an image"""
-
-    log = logging.getLogger(__name__ + ".DeleteImage")
+    """Delete image(s)"""
 
     def get_parser(self, prog_name):
         parser = super(DeleteImage, self).get_parser(prog_name)
         parser.add_argument(
-            "image",
+            "images",
             metavar="<image>",
-            help="Name or ID of image to delete",
+            nargs="+",
+            help="Image(s) to delete (name or ID)",
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
-
         image_client = self.app.client_manager.image
-        image = utils.find_resource(
-            image_client.images,
-            parsed_args.image,
-        )
-        image_client.images.delete(image.id)
+        for image in parsed_args.images:
+            image_obj = utils.find_resource(
+                image_client.images,
+                image,
+            )
+            image_client.images.delete(image_obj.id)
 
 
-class ListImage(lister.Lister):
+class ListImage(command.Lister):
     """List available images"""
 
-    log = logging.getLogger(__name__ + ".ListImage")
-
     def get_parser(self, prog_name):
         parser = super(ListImage, self).get_parser(prog_name)
+        public_group = parser.add_mutually_exclusive_group()
+        public_group.add_argument(
+            "--public",
+            dest="public",
+            action="store_true",
+            default=False,
+            help="List only public images",
+        )
+        public_group.add_argument(
+            "--private",
+            dest="private",
+            action="store_true",
+            default=False,
+            help="List only private images",
+        )
+        # Included for silent CLI compatibility with v2
+        public_group.add_argument(
+            "--shared",
+            dest="shared",
+            action="store_true",
+            default=False,
+            help=argparse.SUPPRESS,
+        )
+        parser.add_argument(
+            '--property',
+            metavar='<key=value>',
+            action=parseractions.KeyValueAction,
+            help='Filter output based on property',
+        )
+        parser.add_argument(
+            '--long',
+            action='store_true',
+            default=False,
+            help='List additional fields in output',
+        )
+
+        # --page-size has never worked, leave here for silent compatibility
+        # We'll implement limit/marker differently later
         parser.add_argument(
             "--page-size",
             metavar="<size>",
-            help="Number of images to request in each paginated request",
+            help=argparse.SUPPRESS,
+        )
+        parser.add_argument(
+            '--sort',
+            metavar="<key>[:<direction>]",
+            help="Sort output by selected keys and directions(asc or desc) "
+                 "(default: asc), multiple keys and directions can be "
+                 "specified separated by comma",
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
-
         image_client = self.app.client_manager.image
 
         kwargs = {}
-        if parsed_args.page_size is not None:
-            kwargs["page_size"] = parsed_args.page_size
+        if parsed_args.public:
+            kwargs['public'] = True
+        if parsed_args.private:
+            kwargs['private'] = True
+        # Note: We specifically need to do that below to get the 'status'
+        #       column.
+        #
+        # Always set kwargs['detailed'] to True, and then filter the columns
+        # according to whether the --long option is specified or not.
+        kwargs['detailed'] = True
 
-        data = image_client.images.list(**kwargs)
-        columns = ["ID", "Name"]
+        if parsed_args.long:
+            columns = (
+                'ID',
+                'Name',
+                'Disk Format',
+                'Container Format',
+                'Size',
+                'Status',
+                'is_public',
+                'protected',
+                'owner',
+                'properties',
+            )
+            column_headers = (
+                'ID',
+                'Name',
+                'Disk Format',
+                'Container Format',
+                'Size',
+                'Status',
+                'Visibility',
+                'Protected',
+                'Project',
+                'Properties',
+            )
+        else:
+            columns = ("ID", "Name", "Status")
+            column_headers = columns
 
-        return (columns, (utils.get_item_properties(s, columns) for s in data))
+        # List of image data received
+        data = []
+        # No pages received yet, so start the page marker at None.
+        marker = None
+        while True:
+            page = image_client.api.image_list(marker=marker, **kwargs)
+            if not page:
+                break
+            data.extend(page)
+            # Set the marker to the id of the last item we received
+            marker = page[-1]['id']
+
+        if parsed_args.property:
+            # NOTE(dtroyer): coerce to a list to subscript it in py3
+            attr, value = list(parsed_args.property.items())[0]
+            api_utils.simple_filter(
+                data,
+                attr=attr,
+                value=value,
+                property_field='properties',
+            )
+
+        data = utils.sort_items(data, parsed_args.sort)
+
+        return (
+            column_headers,
+            (utils.get_dict_properties(
+                s,
+                columns,
+                formatters={
+                    'is_public': _format_visibility,
+                    'properties': utils.format_dict,
+                },
+            ) for s in data)
+        )
 
 
 class SaveImage(command.Command):
     """Save an image locally"""
 
-    log = logging.getLogger(__name__ + ".SaveImage")
-
     def get_parser(self, prog_name):
         parser = super(SaveImage, self).get_parser(prog_name)
         parser.add_argument(
             "--file",
             metavar="<filename>",
-            help="Downloaded image save filename [default: stdout]",
+            help="Downloaded image save filename (default: stdout)",
         )
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Name or ID of image to delete",
+            help="Image to save (name or ID)",
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
-
         image_client = self.app.client_manager.image
         image = utils.find_resource(
             image_client.images,
@@ -340,28 +462,21 @@ class SaveImage(command.Command):
         gc_utils.save_image(data, parsed_args.file)
 
 
-class SetImage(show.ShowOne):
-    """Change image properties"""
-
-    log = logging.getLogger(__name__ + ".SetImage")
+class SetImage(command.Command):
+    """Set image properties"""
 
     def get_parser(self, prog_name):
         parser = super(SetImage, self).get_parser(prog_name)
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Image name or ID to change",
+            help="Image to modify (name or ID)",
         )
         parser.add_argument(
             "--name",
             metavar="<name>",
             help="New image name",
         )
-        parser.add_argument(
-            "--owner",
-            metavar="<project>",
-            help="New image owner project name or ID",
-        )
         parser.add_argument(
             "--min-disk",
             metavar="<disk-gb>",
@@ -374,6 +489,28 @@ class SetImage(show.ShowOne):
             type=int,
             help="Minimum RAM size needed to boot image, in megabytes",
         )
+        container_choices = ["ami", "ari", "aki", "bare", "ovf"]
+        parser.add_argument(
+            "--container-format",
+            metavar="<container-format>",
+            help=("Container format of image. Acceptable formats: %s" %
+                  container_choices),
+            choices=container_choices
+        )
+        disk_choices = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2",
+                        "vdi", "iso"]
+        parser.add_argument(
+            "--disk-format",
+            metavar="<disk-format>",
+            help="Disk format of image. Acceptable formats: %s" % disk_choices,
+            choices=disk_choices
+        )
+        parser.add_argument(
+            "--size",
+            metavar="<size>",
+            type=int,
+            help="Size of image data (in bytes)"
+        )
         protected_group = parser.add_mutually_exclusive_group()
         protected_group.add_argument(
             "--protected",
@@ -401,17 +538,83 @@ class SetImage(show.ShowOne):
             dest="properties",
             metavar="<key=value>",
             action=parseractions.KeyValueAction,
-            help="Set an image property "
+            help="Set a property on this image "
                  "(repeat option to set multiple properties)",
         )
+        parser.add_argument(
+            "--store",
+            metavar="<store>",
+            help="Upload image to this store",
+        )
+        parser.add_argument(
+            "--location",
+            metavar="<image-url>",
+            help="Download image from an existing URL",
+        )
+        parser.add_argument(
+            "--copy-from",
+            metavar="<image-url>",
+            help="Copy image from the data store (similar to --location)",
+        )
+        parser.add_argument(
+            "--file",
+            metavar="<file>",
+            help="Upload image from local file",
+        )
+        parser.add_argument(
+            "--volume",
+            metavar="<volume>",
+            help="Create image from a volume",
+        )
+        parser.add_argument(
+            "--force",
+            dest='force',
+            action='store_true',
+            default=False,
+            help="Force image change if volume is in use "
+            "(only meaningful with --volume)",
+        )
+        parser.add_argument(
+            "--stdin",
+            dest='stdin',
+            action='store_true',
+            default=False,
+            help="Read image data from standard input",
+        )
+        parser.add_argument(
+            "--checksum",
+            metavar="<checksum>",
+            help="Image hash used for verification",
+        )
+        # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
+        #                2.x release.  Do not remove before Jan 2017
+        #                and a 3.x release.
+        project_group = parser.add_mutually_exclusive_group()
+        project_group.add_argument(
+            "--project",
+            metavar="<project>",
+            help="Set an alternate project on this image (name or ID)",
+        )
+        project_group.add_argument(
+            "--owner",
+            metavar="<project>",
+            help=argparse.SUPPRESS,
+        )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
         image_client = self.app.client_manager.image
 
+        if getattr(parsed_args, 'owner', None) is not None:
+            self.log.warning(_(
+                'The --owner option is deprecated, '
+                'please use --project instead.'
+            ))
+
         kwargs = {}
-        copy_attrs = ('name', 'owner', 'min_disk', 'min_ram', 'properties')
+        copy_attrs = ('name', 'owner', 'min_disk', 'min_ram', 'properties',
+                      'container_format', 'disk_format', 'size', 'store',
+                      'location', 'copy_from', 'volume', 'force', 'checksum')
         for attr in copy_attrs:
             if attr in parsed_args:
                 val = getattr(parsed_args, attr, None)
@@ -419,6 +622,12 @@ class SetImage(show.ShowOne):
                     # Only include a value in kwargs for attributes that are
                     # actually present on the command line
                     kwargs[attr] = val
+
+        # Special case project option back to API attribute name 'owner'
+        val = getattr(parsed_args, 'project', None)
+        if val:
+            kwargs['owner'] = val
+
         # Handle exclusive booleans with care
         # Avoid including attributes in kwargs if an option is not
         # present on the command line.  These exclusive booleans are not
@@ -434,43 +643,77 @@ class SetImage(show.ShowOne):
         if parsed_args.private:
             kwargs['is_public'] = False
 
-        if not kwargs:
-            self.log.warning('no arguments specified')
-            return {}, {}
+        # Wrap the call to catch exceptions in order to close files
+        try:
+            image = utils.find_resource(
+                image_client.images,
+                parsed_args.image,
+            )
 
-        image = utils.find_resource(
-            image_client.images,
-            parsed_args.image,
-        )
+            if not parsed_args.location and not parsed_args.copy_from:
+                if parsed_args.volume:
+                    volume_client = self.app.client_manager.volume
+                    source_volume = utils.find_resource(
+                        volume_client.volumes,
+                        parsed_args.volume,
+                    )
+                    volume_client.volumes.upload_to_image(
+                        source_volume.id,
+                        parsed_args.force,
+                        parsed_args.image,
+                        (parsed_args.container_format
+                         if parsed_args.container_format
+                         else image.container_format),
+                        (parsed_args.disk_format
+                         if parsed_args.disk_format
+                         else image.disk_format),
+                    )
+                elif parsed_args.file:
+                    # Send an open file handle to glanceclient so it will
+                    # do a chunked transfer
+                    kwargs["data"] = io.open(parsed_args.file, "rb")
+                else:
+                    # Read file from stdin
+                    if sys.stdin.isatty() is not True:
+                        if parsed_args.stdin:
+                            if msvcrt:
+                                msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
+                            # Send an open file handle to glanceclient so it
+                            # will do a chunked transfer
+                            kwargs["data"] = sys.stdin
+                        else:
+                            self.log.warning('Use --stdin to enable read image'
+                                             ' data from standard input')
 
-        if image.properties and parsed_args.properties:
-            image.properties.update(kwargs['properties'])
-            kwargs['properties'] = image.properties
+            if image.properties and parsed_args.properties:
+                image.properties.update(kwargs['properties'])
+                kwargs['properties'] = image.properties
 
-        image = image_client.images.update(image.id, **kwargs)
+            if not kwargs:
+                self.log.warning('no arguments specified')
+                return
 
-        info = {}
-        info.update(image._info)
-        return zip(*sorted(six.iteritems(info)))
+            image = image_client.images.update(image.id, **kwargs)
+        finally:
+            # Clean up open files - make sure data isn't a string
+            if ('data' in kwargs and hasattr(kwargs['data'], 'close') and
+                    kwargs['data'] != sys.stdin):
+                kwargs['data'].close()
 
 
-class ShowImage(show.ShowOne):
-    """Show image details"""
-
-    log = logging.getLogger(__name__ + ".ShowImage")
+class ShowImage(command.ShowOne):
+    """Display image details"""
 
     def get_parser(self, prog_name):
         parser = super(ShowImage, self).get_parser(prog_name)
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Name or ID of image to display",
+            help="Image to display (name or ID)",
         )
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)", parsed_args)
-
         image_client = self.app.client_manager.image
         image = utils.find_resource(
             image_client.images,
@@ -479,4 +722,5 @@ class ShowImage(show.ShowOne):
 
         info = {}
         info.update(image._info)
+        info['properties'] = utils.format_dict(info.get('properties', {}))
         return zip(*sorted(six.iteritems(info)))
