@@ -37,6 +37,17 @@ class TestEndpoint(identity_fakes.TestIdentityv3):
 
 class TestEndpointCreate(TestEndpoint):
 
+    columns = (
+        'enabled',
+        'id',
+        'interface',
+        'region',
+        'service_id',
+        'service_name',
+        'service_type',
+        'url',
+    )
+
     def setUp(self):
         super(TestEndpointCreate, self).setUp()
 
@@ -70,7 +81,9 @@ class TestEndpointCreate(TestEndpoint):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -86,9 +99,7 @@ class TestEndpointCreate(TestEndpoint):
             **kwargs
         )
 
-        collist = ('enabled', 'id', 'interface', 'region', 'service_id',
-                   'service_name', 'service_type', 'url')
-        self.assertEqual(collist, columns)
+        self.assertEqual(self.columns, columns)
         datalist = (
             True,
             identity_fakes.endpoint_id,
@@ -117,7 +128,9 @@ class TestEndpointCreate(TestEndpoint):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -133,9 +146,7 @@ class TestEndpointCreate(TestEndpoint):
             **kwargs
         )
 
-        collist = ('enabled', 'id', 'interface', 'region', 'service_id',
-                   'service_name', 'service_type', 'url')
-        self.assertEqual(collist, columns)
+        self.assertEqual(self.columns, columns)
         datalist = (
             True,
             identity_fakes.endpoint_id,
@@ -163,7 +174,9 @@ class TestEndpointCreate(TestEndpoint):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -179,9 +192,7 @@ class TestEndpointCreate(TestEndpoint):
             **kwargs
         )
 
-        collist = ('enabled', 'id', 'interface', 'region', 'service_id',
-                   'service_name', 'service_type', 'url')
-        self.assertEqual(collist, columns)
+        self.assertEqual(self.columns, columns)
         datalist = (
             True,
             identity_fakes.endpoint_id,
@@ -209,7 +220,9 @@ class TestEndpointCreate(TestEndpoint):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -225,9 +238,7 @@ class TestEndpointCreate(TestEndpoint):
             **kwargs
         )
 
-        collist = ('enabled', 'id', 'interface', 'region', 'service_id',
-                   'service_name', 'service_type', 'url')
-        self.assertEqual(collist, columns)
+        self.assertEqual(self.columns, columns)
         datalist = (
             True,
             identity_fakes.endpoint_id,
@@ -276,6 +287,16 @@ class TestEndpointDelete(TestEndpoint):
 
 class TestEndpointList(TestEndpoint):
 
+    columns = (
+        'ID',
+        'Region',
+        'Service Name',
+        'Service Type',
+        'Enabled',
+        'Interface',
+        'URL',
+    )
+
     def setUp(self):
         super(TestEndpointList, self).setUp()
 
@@ -302,22 +323,24 @@ class TestEndpointList(TestEndpoint):
         verifylist = []
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
         self.endpoints_mock.list.assert_called_with()
 
-        collist = ('ID', 'Region', 'Service Name', 'Service Type',
-                   'Enabled', 'Interface', 'URL')
-        self.assertEqual(collist, columns)
-        datalist = ((
-            identity_fakes.endpoint_id,
-            identity_fakes.endpoint_region,
-            self.get_fake_service_name(),
-            identity_fakes.service_type,
-            True,
-            identity_fakes.endpoint_interface,
-            identity_fakes.endpoint_url,
-        ),)
+        self.assertEqual(self.columns, columns)
+        datalist = (
+            (
+                identity_fakes.endpoint_id,
+                identity_fakes.endpoint_region,
+                self.get_fake_service_name(),
+                identity_fakes.service_type,
+                True,
+                identity_fakes.endpoint_interface,
+                identity_fakes.endpoint_url,
+            ),
+        )
         self.assertEqual(datalist, tuple(data))
 
     def test_endpoint_list_service(self):
@@ -329,7 +352,9 @@ class TestEndpointList(TestEndpoint):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -338,18 +363,18 @@ class TestEndpointList(TestEndpoint):
         }
         self.endpoints_mock.list.assert_called_with(**kwargs)
 
-        collist = ('ID', 'Region', 'Service Name', 'Service Type',
-                   'Enabled', 'Interface', 'URL')
-        self.assertEqual(collist, columns)
-        datalist = ((
-            identity_fakes.endpoint_id,
-            identity_fakes.endpoint_region,
-            self.get_fake_service_name(),
-            identity_fakes.service_type,
-            True,
-            identity_fakes.endpoint_interface,
-            identity_fakes.endpoint_url,
-        ),)
+        self.assertEqual(self.columns, columns)
+        datalist = (
+            (
+                identity_fakes.endpoint_id,
+                identity_fakes.endpoint_region,
+                self.get_fake_service_name(),
+                identity_fakes.service_type,
+                True,
+                identity_fakes.endpoint_interface,
+                identity_fakes.endpoint_url,
+            ),
+        )
         self.assertEqual(datalist, tuple(data))
 
     def test_endpoint_list_interface(self):
@@ -361,7 +386,9 @@ class TestEndpointList(TestEndpoint):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -370,18 +397,18 @@ class TestEndpointList(TestEndpoint):
         }
         self.endpoints_mock.list.assert_called_with(**kwargs)
 
-        collist = ('ID', 'Region', 'Service Name', 'Service Type',
-                   'Enabled', 'Interface', 'URL')
-        self.assertEqual(collist, columns)
-        datalist = ((
-            identity_fakes.endpoint_id,
-            identity_fakes.endpoint_region,
-            self.get_fake_service_name(),
-            identity_fakes.service_type,
-            True,
-            identity_fakes.endpoint_interface,
-            identity_fakes.endpoint_url,
-        ),)
+        self.assertEqual(self.columns, columns)
+        datalist = (
+            (
+                identity_fakes.endpoint_id,
+                identity_fakes.endpoint_region,
+                self.get_fake_service_name(),
+                identity_fakes.service_type,
+                True,
+                identity_fakes.endpoint_interface,
+                identity_fakes.endpoint_url,
+            ),
+        )
         self.assertEqual(datalist, tuple(data))
 
     def test_endpoint_list_region(self):
@@ -393,7 +420,9 @@ class TestEndpointList(TestEndpoint):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -402,18 +431,18 @@ class TestEndpointList(TestEndpoint):
         }
         self.endpoints_mock.list.assert_called_with(**kwargs)
 
-        collist = ('ID', 'Region', 'Service Name', 'Service Type',
-                   'Enabled', 'Interface', 'URL')
-        self.assertEqual(collist, columns)
-        datalist = ((
-            identity_fakes.endpoint_id,
-            identity_fakes.endpoint_region,
-            self.get_fake_service_name(),
-            identity_fakes.service_type,
-            True,
-            identity_fakes.endpoint_interface,
-            identity_fakes.endpoint_url,
-        ),)
+        self.assertEqual(self.columns, columns)
+        datalist = (
+            (
+                identity_fakes.endpoint_id,
+                identity_fakes.endpoint_region,
+                self.get_fake_service_name(),
+                identity_fakes.service_type,
+                True,
+                identity_fakes.endpoint_interface,
+                identity_fakes.endpoint_url,
+            ),
+        )
         self.assertEqual(datalist, tuple(data))
 
 
@@ -652,14 +681,24 @@ class TestEndpointShow(TestEndpoint):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
         self.endpoints_mock.get.assert_called_with(
             identity_fakes.endpoint_id,
         )
 
-        collist = ('enabled', 'id', 'interface', 'region', 'service_id',
-                   'service_name', 'service_type', 'url')
+        collist = (
+            'enabled',
+            'id',
+            'interface',
+            'region',
+            'service_id',
+            'service_name',
+            'service_type',
+            'url',
+        )
         self.assertEqual(collist, columns)
         datalist = (
             True,
