@@ -38,6 +38,17 @@ class TestGroup(identity_fakes.TestIdentityv3):
 
 class TestGroupList(TestGroup):
 
+    columns = (
+        'ID',
+        'Name',
+    )
+    datalist = (
+        (
+            identity_fakes.group_id,
+            identity_fakes.group_name,
+        ),
+    )
+
     def setUp(self):
         super(TestGroupList, self).setUp()
 
@@ -74,7 +85,9 @@ class TestGroupList(TestGroup):
         verifylist = []
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -87,13 +100,8 @@ class TestGroupList(TestGroup):
             **kwargs
         )
 
-        collist = ('ID', 'Name')
-        self.assertEqual(collist, columns)
-        datalist = ((
-            identity_fakes.group_id,
-            identity_fakes.group_name,
-        ), )
-        self.assertEqual(datalist, tuple(data))
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
 
     def test_group_list_domain(self):
         arglist = [
@@ -104,7 +112,9 @@ class TestGroupList(TestGroup):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -117,13 +127,8 @@ class TestGroupList(TestGroup):
             **kwargs
         )
 
-        collist = ('ID', 'Name')
-        self.assertEqual(collist, columns)
-        datalist = ((
-            identity_fakes.group_id,
-            identity_fakes.group_name,
-        ), )
-        self.assertEqual(datalist, tuple(data))
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
 
     def test_group_list_user(self):
         arglist = [
@@ -134,7 +139,9 @@ class TestGroupList(TestGroup):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -147,13 +154,8 @@ class TestGroupList(TestGroup):
             **kwargs
         )
 
-        collist = ('ID', 'Name')
-        self.assertEqual(collist, columns)
-        datalist = ((
-            identity_fakes.group_id,
-            identity_fakes.group_name,
-        ), )
-        self.assertEqual(datalist, tuple(data))
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
 
     def test_group_list_long(self):
         arglist = [
@@ -164,7 +166,9 @@ class TestGroupList(TestGroup):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -177,17 +181,15 @@ class TestGroupList(TestGroup):
             **kwargs
         )
 
-        collist = (
-            'ID',
-            'Name',
+        columns = self.columns + (
             'Domain ID',
             'Description',
         )
-        self.assertEqual(collist, columns)
         datalist = ((
             identity_fakes.group_id,
             identity_fakes.group_name,
             '',
             '',
         ), )
+        self.assertEqual(columns, columns)
         self.assertEqual(datalist, tuple(data))
