@@ -45,6 +45,15 @@ class TestRole(identity_fakes.TestIdentityv3):
         self.roles_mock = self.app.client_manager.identity.roles
         self.roles_mock.reset_mock()
 
+    def _is_inheritance_testcase(self):
+        return False
+
+
+class TestRoleInherited(TestRole):
+
+    def _is_inheritance_testcase(self):
+        return True
+
 
 class TestRoleAdd(TestRole):
 
@@ -95,22 +104,26 @@ class TestRoleAdd(TestRole):
             '--domain', identity_fakes.domain_name,
             identity_fakes.role_name,
         ]
+        if self._is_inheritance_testcase():
+            arglist.append('--inherited')
         verifylist = [
             ('user', identity_fakes.user_name),
             ('group', None),
             ('domain', identity_fakes.domain_name),
             ('project', None),
             ('role', identity_fakes.role_name),
+            ('inherited', self._is_inheritance_testcase()),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.run(parsed_args)
-        self.assertEqual(result, 0)
+        self.assertEqual(0, result)
 
         # Set expected values
         kwargs = {
             'user': identity_fakes.user_id,
             'domain': identity_fakes.domain_id,
+            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
         }
         # RoleManager.grant(role, user=, group=, domain=, project=)
         self.roles_mock.grant.assert_called_with(
@@ -124,22 +137,26 @@ class TestRoleAdd(TestRole):
             '--project', identity_fakes.project_name,
             identity_fakes.role_name,
         ]
+        if self._is_inheritance_testcase():
+            arglist.append('--inherited')
         verifylist = [
             ('user', identity_fakes.user_name),
             ('group', None),
             ('domain', None),
             ('project', identity_fakes.project_name),
             ('role', identity_fakes.role_name),
+            ('inherited', self._is_inheritance_testcase()),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.run(parsed_args)
-        self.assertEqual(result, 0)
+        self.assertEqual(0, result)
 
         # Set expected values
         kwargs = {
             'user': identity_fakes.user_id,
             'project': identity_fakes.project_id,
+            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
         }
         # RoleManager.grant(role, user=, group=, domain=, project=)
         self.roles_mock.grant.assert_called_with(
@@ -153,22 +170,26 @@ class TestRoleAdd(TestRole):
             '--domain', identity_fakes.domain_name,
             identity_fakes.role_name,
         ]
+        if self._is_inheritance_testcase():
+            arglist.append('--inherited')
         verifylist = [
             ('user', None),
             ('group', identity_fakes.group_name),
             ('domain', identity_fakes.domain_name),
             ('project', None),
             ('role', identity_fakes.role_name),
+            ('inherited', self._is_inheritance_testcase()),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.run(parsed_args)
-        self.assertEqual(result, 0)
+        self.assertEqual(0, result)
 
         # Set expected values
         kwargs = {
             'group': identity_fakes.group_id,
             'domain': identity_fakes.domain_id,
+            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
         }
         # RoleManager.grant(role, user=, group=, domain=, project=)
         self.roles_mock.grant.assert_called_with(
@@ -182,22 +203,26 @@ class TestRoleAdd(TestRole):
             '--project', identity_fakes.project_name,
             identity_fakes.role_name,
         ]
+        if self._is_inheritance_testcase():
+            arglist.append('--inherited')
         verifylist = [
             ('user', None),
             ('group', identity_fakes.group_name),
             ('domain', None),
             ('project', identity_fakes.project_name),
             ('role', identity_fakes.role_name),
+            ('inherited', self._is_inheritance_testcase()),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.run(parsed_args)
-        self.assertEqual(result, 0)
+        self.assertEqual(0, result)
 
         # Set expected values
         kwargs = {
             'group': identity_fakes.group_id,
             'project': identity_fakes.project_id,
+            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
         }
         # RoleManager.grant(role, user=, group=, domain=, project=)
         self.roles_mock.grant.assert_called_with(
@@ -206,6 +231,10 @@ class TestRoleAdd(TestRole):
         )
 
 
+class TestRoleAddInherited(TestRoleAdd, TestRoleInherited):
+    pass
+
+
 class TestRoleCreate(TestRole):
 
     def setUp(self):
@@ -229,7 +258,9 @@ class TestRoleCreate(TestRole):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -243,12 +274,12 @@ class TestRoleCreate(TestRole):
         )
 
         collist = ('id', 'name')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.role_id,
             identity_fakes.role_name,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
 
 
 class TestRoleDelete(TestRole):
@@ -271,11 +302,10 @@ class TestRoleDelete(TestRole):
             identity_fakes.role_name,
         ]
         verifylist = [
-            ('role', identity_fakes.role_name),
+            ('roles', [identity_fakes.role_name]),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         self.roles_mock.delete.assert_called_with(
@@ -285,6 +315,17 @@ class TestRoleDelete(TestRole):
 
 class TestRoleList(TestRole):
 
+    columns = (
+        'ID',
+        'Name',
+    )
+    datalist = (
+        (
+            identity_fakes.role_id,
+            identity_fakes.role_name,
+        ),
+    )
+
     def setUp(self):
         super(TestRoleList, self).setUp()
 
@@ -325,18 +366,45 @@ class TestRoleList(TestRole):
         verifylist = []
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         self.roles_mock.list.assert_called_with()
 
-        collist = ('ID', 'Name')
-        self.assertEqual(columns, collist)
-        datalist = ((
-            identity_fakes.role_id,
-            identity_fakes.role_name,
-        ), )
-        self.assertEqual(tuple(data), datalist)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
+    def test_user_list_inherited(self):
+        arglist = [
+            '--user', identity_fakes.user_id,
+            '--inherited',
+        ]
+        verifylist = [
+            ('user', identity_fakes.user_id),
+            ('inherited', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
+        columns, data = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = {
+            'domain': 'default',
+            'user': self.users_mock.get(),
+            'os_inherit_extension_inherited': True,
+        }
+        # RoleManager.list(user=, group=, domain=, project=, **kwargs)
+        self.roles_mock.list.assert_called_with(
+            **kwargs
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
 
     def test_user_list_user(self):
         arglist = [
@@ -347,26 +415,24 @@ class TestRoleList(TestRole):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'domain': 'default',
             'user': self.users_mock.get(),
+            'os_inherit_extension_inherited': False
         }
         # RoleManager.list(user=, group=, domain=, project=, **kwargs)
         self.roles_mock.list.assert_called_with(
             **kwargs
         )
 
-        collist = ('ID', 'Name')
-        self.assertEqual(columns, collist)
-        datalist = ((
-            identity_fakes.role_id,
-            identity_fakes.role_name,
-        ), )
-        self.assertEqual(tuple(data), datalist)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
 
     def test_role_list_domain_user(self):
         arglist = [
@@ -379,13 +445,16 @@ class TestRoleList(TestRole):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'domain': self.domains_mock.get(),
             'user': self.users_mock.get(),
+            'os_inherit_extension_inherited': False
         }
         # RoleManager.list(user=, group=, domain=, project=, **kwargs)
         self.roles_mock.list.assert_called_with(
@@ -393,14 +462,14 @@ class TestRoleList(TestRole):
         )
 
         collist = ('ID', 'Name', 'Domain', 'User')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = ((
             identity_fakes.role_id,
             identity_fakes.role_name,
             identity_fakes.domain_name,
             identity_fakes.user_name,
         ), )
-        self.assertEqual(tuple(data), datalist)
+        self.assertEqual(datalist, tuple(data))
 
     def test_role_list_domain_group(self):
         arglist = [
@@ -413,13 +482,16 @@ class TestRoleList(TestRole):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'domain': self.domains_mock.get(),
             'group': self.groups_mock.get(),
+            'os_inherit_extension_inherited': False
         }
         # RoleManager.list(user=, group=, domain=, project=, **kwargs)
         self.roles_mock.list.assert_called_with(
@@ -427,14 +499,14 @@ class TestRoleList(TestRole):
         )
 
         collist = ('ID', 'Name', 'Domain', 'Group')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = ((
             identity_fakes.role_id,
             identity_fakes.role_name,
             identity_fakes.domain_name,
             identity_fakes.group_name,
         ), )
-        self.assertEqual(tuple(data), datalist)
+        self.assertEqual(datalist, tuple(data))
 
     def test_role_list_project_user(self):
         arglist = [
@@ -447,13 +519,16 @@ class TestRoleList(TestRole):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'project': self.projects_mock.get(),
             'user': self.users_mock.get(),
+            'os_inherit_extension_inherited': False
         }
         # RoleManager.list(user=, group=, domain=, project=, **kwargs)
         self.roles_mock.list.assert_called_with(
@@ -461,14 +536,14 @@ class TestRoleList(TestRole):
         )
 
         collist = ('ID', 'Name', 'Project', 'User')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = ((
             identity_fakes.role_id,
             identity_fakes.role_name,
             identity_fakes.project_name,
             identity_fakes.user_name,
         ), )
-        self.assertEqual(tuple(data), datalist)
+        self.assertEqual(datalist, tuple(data))
 
     def test_role_list_project_group(self):
         arglist = [
@@ -481,13 +556,16 @@ class TestRoleList(TestRole):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'project': self.projects_mock.get(),
             'group': self.groups_mock.get(),
+            'os_inherit_extension_inherited': False
         }
         # RoleManager.list(user=, group=, domain=, project=, **kwargs)
         self.roles_mock.list.assert_called_with(
@@ -495,14 +573,14 @@ class TestRoleList(TestRole):
         )
 
         collist = ('ID', 'Name', 'Project', 'Group')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = ((
             identity_fakes.role_id,
             identity_fakes.role_name,
             identity_fakes.project_name,
             identity_fakes.group_name,
         ), )
-        self.assertEqual(tuple(data), datalist)
+        self.assertEqual(datalist, tuple(data))
 
 
 class TestRoleRemove(TestRole):
@@ -550,22 +628,25 @@ class TestRoleRemove(TestRole):
             '--domain', identity_fakes.domain_name,
             identity_fakes.role_name,
         ]
+        if self._is_inheritance_testcase():
+            arglist.append('--inherited')
         verifylist = [
             ('user', identity_fakes.user_name),
             ('group', None),
             ('domain', identity_fakes.domain_name),
             ('project', None),
             ('role', identity_fakes.role_name),
+            ('inherited', self._is_inheritance_testcase()),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'user': identity_fakes.user_id,
             'domain': identity_fakes.domain_id,
+            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
         }
         # RoleManager.revoke(role, user=, group=, domain=, project=)
         self.roles_mock.revoke.assert_called_with(
@@ -579,22 +660,25 @@ class TestRoleRemove(TestRole):
             '--project', identity_fakes.project_name,
             identity_fakes.role_name,
         ]
+        if self._is_inheritance_testcase():
+            arglist.append('--inherited')
         verifylist = [
             ('user', identity_fakes.user_name),
             ('group', None),
             ('domain', None),
             ('project', identity_fakes.project_name),
             ('role', identity_fakes.role_name),
+            ('inherited', self._is_inheritance_testcase()),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'user': identity_fakes.user_id,
             'project': identity_fakes.project_id,
+            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
         }
         # RoleManager.revoke(role, user=, group=, domain=, project=)
         self.roles_mock.revoke.assert_called_with(
@@ -608,22 +692,26 @@ class TestRoleRemove(TestRole):
             '--domain', identity_fakes.domain_name,
             identity_fakes.role_name,
         ]
+        if self._is_inheritance_testcase():
+            arglist.append('--inherited')
         verifylist = [
             ('user', None),
             ('group', identity_fakes.group_name),
             ('domain', identity_fakes.domain_name),
             ('project', None),
             ('role', identity_fakes.role_name),
+            ('role', identity_fakes.role_name),
+            ('inherited', self._is_inheritance_testcase()),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'group': identity_fakes.group_id,
             'domain': identity_fakes.domain_id,
+            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
         }
         # RoleManager.revoke(role, user=, group=, domain=, project=)
         self.roles_mock.revoke.assert_called_with(
@@ -637,22 +725,25 @@ class TestRoleRemove(TestRole):
             '--project', identity_fakes.project_name,
             identity_fakes.role_name,
         ]
+        if self._is_inheritance_testcase():
+            arglist.append('--inherited')
         verifylist = [
             ('user', None),
             ('group', identity_fakes.group_name),
             ('domain', None),
             ('project', identity_fakes.project_name),
             ('role', identity_fakes.role_name),
+            ('inherited', self._is_inheritance_testcase()),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'group': identity_fakes.group_id,
             'project': identity_fakes.project_id,
+            'os_inherit_extension_inherited': self._is_inheritance_testcase(),
         }
         # RoleManager.revoke(role, user=, group=, domain=, project=)
         self.roles_mock.revoke.assert_called_with(
@@ -687,7 +778,6 @@ class TestRoleSet(TestRole):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -715,7 +805,7 @@ class TestRoleShow(TestRole):
         # Get the command object to test
         self.cmd = role.ShowRole(self.app, None)
 
-    def test_service_show(self):
+    def test_role_show(self):
         arglist = [
             identity_fakes.role_name,
         ]
@@ -724,7 +814,9 @@ class TestRoleShow(TestRole):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # RoleManager.get(role)
@@ -733,9 +825,9 @@ class TestRoleShow(TestRole):
         )
 
         collist = ('id', 'name')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.role_id,
             identity_fakes.role_name,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
