@@ -13,7 +13,9 @@
 #   under the License.
 #
 
+import contextlib
 import copy
+
 import mock
 
 from openstackclient.identity.v3 import user
@@ -42,9 +44,31 @@ class TestUser(identity_fakes.TestIdentityv3):
         self.users_mock = self.app.client_manager.identity.users
         self.users_mock.reset_mock()
 
+        # Shortcut for RoleAssignmentManager Mock
+        self.role_assignments_mock = self.app.client_manager.identity.\
+            role_assignments
+        self.role_assignments_mock.reset_mock()
+
 
 class TestUserCreate(TestUser):
 
+    columns = (
+        'default_project_id',
+        'domain_id',
+        'email',
+        'enabled',
+        'id',
+        'name',
+    )
+    datalist = (
+        identity_fakes.project_id,
+        identity_fakes.domain_id,
+        identity_fakes.user_email,
+        True,
+        identity_fakes.user_id,
+        identity_fakes.user_name,
+    )
+
     def setUp(self):
         super(TestUserCreate, self).setUp()
 
@@ -80,7 +104,9 @@ class TestUserCreate(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -100,17 +126,8 @@ class TestUserCreate(TestUser):
             **kwargs
         )
 
-        collist = ('domain_id', 'email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
-        datalist = (
-            identity_fakes.domain_id,
-            identity_fakes.user_email,
-            True,
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-            identity_fakes.project_id,
-        )
-        self.assertEqual(data, datalist)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
 
     def test_user_create_password(self):
         arglist = [
@@ -126,7 +143,9 @@ class TestUserCreate(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -144,18 +163,8 @@ class TestUserCreate(TestUser):
         self.users_mock.create.assert_called_with(
             **kwargs
         )
-
-        collist = ('domain_id', 'email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
-        datalist = (
-            identity_fakes.domain_id,
-            identity_fakes.user_email,
-            True,
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-            identity_fakes.project_id,
-        )
-        self.assertEqual(data, datalist)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
 
     def test_user_create_password_prompt(self):
         arglist = [
@@ -171,7 +180,9 @@ class TestUserCreate(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         mocker = mock.Mock()
         mocker.return_value = 'abc123'
         with mock.patch("openstackclient.common.utils.get_password", mocker):
@@ -193,17 +204,8 @@ class TestUserCreate(TestUser):
             **kwargs
         )
 
-        collist = ('domain_id', 'email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
-        datalist = (
-            identity_fakes.domain_id,
-            identity_fakes.user_email,
-            True,
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-            identity_fakes.project_id,
-        )
-        self.assertEqual(data, datalist)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
 
     def test_user_create_email(self):
         arglist = [
@@ -218,7 +220,9 @@ class TestUserCreate(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -237,17 +241,8 @@ class TestUserCreate(TestUser):
             **kwargs
         )
 
-        collist = ('domain_id', 'email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
-        datalist = (
-            identity_fakes.domain_id,
-            identity_fakes.user_email,
-            True,
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-            identity_fakes.project_id,
-        )
-        self.assertEqual(data, datalist)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
 
     def test_user_create_project(self):
         # Return the new project
@@ -258,7 +253,7 @@ class TestUserCreate(TestUser):
         )
         # Set up to return an updated user
         USER_2 = copy.deepcopy(identity_fakes.USER)
-        USER_2['project_id'] = identity_fakes.PROJECT_2['id']
+        USER_2['default_project_id'] = identity_fakes.PROJECT_2['id']
         self.users_mock.create.return_value = fakes.FakeResource(
             None,
             USER_2,
@@ -277,7 +272,9 @@ class TestUserCreate(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -296,17 +293,78 @@ class TestUserCreate(TestUser):
             **kwargs
         )
 
-        collist = ('domain_id', 'email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        self.assertEqual(self.columns, columns)
         datalist = (
+            identity_fakes.PROJECT_2['id'],
             identity_fakes.domain_id,
             identity_fakes.user_email,
             True,
             identity_fakes.user_id,
             identity_fakes.user_name,
-            identity_fakes.PROJECT_2['id'],
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
+
+    def test_user_create_project_domain(self):
+        # Return the new project
+        self.projects_mock.get.return_value = fakes.FakeResource(
+            None,
+            copy.deepcopy(identity_fakes.PROJECT_2),
+            loaded=True,
+        )
+        # Set up to return an updated user
+        USER_2 = copy.deepcopy(identity_fakes.USER)
+        USER_2['default_project_id'] = identity_fakes.PROJECT_2['id']
+        self.users_mock.create.return_value = fakes.FakeResource(
+            None,
+            USER_2,
+            loaded=True,
+        )
+
+        arglist = [
+            '--project', identity_fakes.PROJECT_2['name'],
+            '--project-domain', identity_fakes.PROJECT_2['domain_id'],
+            identity_fakes.user_name,
+        ]
+        verifylist = [
+            ('project', identity_fakes.PROJECT_2['name']),
+            ('project_domain', identity_fakes.PROJECT_2['domain_id']),
+            ('enable', False),
+            ('disable', False),
+            ('name', identity_fakes.user_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
+        columns, data = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = {
+            'name': identity_fakes.user_name,
+            'default_project': identity_fakes.PROJECT_2['id'],
+            'description': None,
+            'domain': None,
+            'email': None,
+            'enabled': True,
+            'password': None,
+        }
+        # UserManager.create(name=, domain=, project=, password=, email=,
+        #   description=, enabled=, default_project=)
+        self.users_mock.create.assert_called_with(
+            **kwargs
+        )
+
+        self.assertEqual(self.columns, columns)
+        datalist = (
+            identity_fakes.PROJECT_2['id'],
+            identity_fakes.domain_id,
+            identity_fakes.user_email,
+            True,
+            identity_fakes.user_id,
+            identity_fakes.user_name,
+        )
+        self.assertEqual(datalist, data)
 
     def test_user_create_domain(self):
         arglist = [
@@ -321,7 +379,9 @@ class TestUserCreate(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -340,17 +400,8 @@ class TestUserCreate(TestUser):
             **kwargs
         )
 
-        collist = ('domain_id', 'email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
-        datalist = (
-            identity_fakes.domain_id,
-            identity_fakes.user_email,
-            True,
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-            identity_fakes.project_id,
-        )
-        self.assertEqual(data, datalist)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
 
     def test_user_create_enable(self):
         arglist = [
@@ -364,7 +415,9 @@ class TestUserCreate(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -383,17 +436,8 @@ class TestUserCreate(TestUser):
             **kwargs
         )
 
-        collist = ('domain_id', 'email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
-        datalist = (
-            identity_fakes.domain_id,
-            identity_fakes.user_email,
-            True,
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-            identity_fakes.project_id,
-        )
-        self.assertEqual(data, datalist)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
 
     def test_user_create_disable(self):
         arglist = [
@@ -407,7 +451,9 @@ class TestUserCreate(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -424,18 +470,8 @@ class TestUserCreate(TestUser):
         self.users_mock.create.assert_called_with(
             **kwargs
         )
-
-        collist = ('domain_id', 'email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
-        datalist = (
-            identity_fakes.domain_id,
-            identity_fakes.user_email,
-            True,
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-            identity_fakes.project_id,
-        )
-        self.assertEqual(data, datalist)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
 
 
 class TestUserDelete(TestUser):
@@ -459,11 +495,10 @@ class TestUserDelete(TestUser):
             identity_fakes.user_id,
         ]
         verifylist = [
-            ('user', identity_fakes.user_id),
+            ('users', [identity_fakes.user_id]),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         self.users_mock.delete.assert_called_with(
@@ -473,6 +508,17 @@ class TestUserDelete(TestUser):
 
 class TestUserList(TestUser):
 
+    columns = [
+        'ID',
+        'Name'
+    ]
+    datalist = (
+        (
+            identity_fakes.user_id,
+            identity_fakes.user_name,
+        ),
+    )
+
     def setUp(self):
         super(TestUserList, self).setUp()
 
@@ -501,6 +547,21 @@ class TestUserList(TestUser):
             loaded=True,
         )
 
+        self.projects_mock.get.return_value = fakes.FakeResource(
+            None,
+            copy.deepcopy(identity_fakes.PROJECT),
+            loaded=True,
+        )
+
+        self.role_assignments_mock.list.return_value = [
+            fakes.FakeResource(
+                None,
+                copy.deepcopy(
+                    identity_fakes.ASSIGNMENT_WITH_PROJECT_ID_AND_USER_ID),
+                loaded=True,
+            )
+        ]
+
         # Get the command object to test
         self.cmd = user.ListUser(self.app, None)
 
@@ -509,7 +570,9 @@ class TestUserList(TestUser):
         verifylist = []
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -522,13 +585,8 @@ class TestUserList(TestUser):
             **kwargs
         )
 
-        collist = ('ID', 'Name')
-        self.assertEqual(columns, collist)
-        datalist = ((
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-        ), )
-        self.assertEqual(datalist, tuple(data))
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
 
     def test_user_list_domain(self):
         arglist = [
@@ -539,7 +597,9 @@ class TestUserList(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -552,13 +612,8 @@ class TestUserList(TestUser):
             **kwargs
         )
 
-        collist = ('ID', 'Name')
-        self.assertEqual(columns, collist)
-        datalist = ((
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-        ), )
-        self.assertEqual(datalist, tuple(data))
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
 
     def test_user_list_group(self):
         arglist = [
@@ -569,7 +624,9 @@ class TestUserList(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -582,13 +639,8 @@ class TestUserList(TestUser):
             **kwargs
         )
 
-        collist = ('ID', 'Name')
-        self.assertEqual(columns, collist)
-        datalist = ((
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-        ), )
-        self.assertEqual(datalist, tuple(data))
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
 
     def test_user_list_long(self):
         arglist = [
@@ -599,7 +651,9 @@ class TestUserList(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -612,39 +666,59 @@ class TestUserList(TestUser):
             **kwargs
         )
 
-        collist = (
+        collist = [
             'ID',
             'Name',
-            'Project Id',
-            'Domain Id',
+            'Project',
+            'Domain',
             'Description',
             'Email',
             'Enabled',
+        ]
+        self.assertEqual(collist, columns)
+        datalist = (
+            (
+                identity_fakes.user_id,
+                identity_fakes.user_name,
+                identity_fakes.project_id,
+                identity_fakes.domain_id,
+                '',
+                identity_fakes.user_email,
+                True,
+            ),
         )
-        self.assertEqual(columns, collist)
-        datalist = ((
-            identity_fakes.user_id,
-            identity_fakes.user_name,
-            identity_fakes.project_id,
-            identity_fakes.domain_id,
-            '',
-            identity_fakes.user_email,
-            True,
-        ), )
         self.assertEqual(datalist, tuple(data))
 
+    def test_user_list_project(self):
+        arglist = [
+            '--project', identity_fakes.project_name,
+        ]
+        verifylist = [
+            ('project', identity_fakes.project_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
+        columns, data = self.cmd.take_action(parsed_args)
+
+        kwargs = {
+            'project': identity_fakes.project_id,
+        }
+
+        self.role_assignments_mock.list.assert_called_with(**kwargs)
+        self.users_mock.get.assert_called_with(identity_fakes.user_id)
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
 
 class TestUserSet(TestUser):
 
     def setUp(self):
         super(TestUserSet, self).setUp()
 
-        self.domains_mock.get.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(identity_fakes.DOMAIN),
-            loaded=True,
-        )
-
         self.projects_mock.get.return_value = fakes.FakeResource(
             None,
             copy.deepcopy(identity_fakes.PROJECT),
@@ -673,7 +747,6 @@ class TestUserSet(TestUser):
             ('name', None),
             ('password', None),
             ('email', None),
-            ('domain', None),
             ('project', None),
             ('enable', False),
             ('disable', False),
@@ -682,7 +755,7 @@ class TestUserSet(TestUser):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.run(parsed_args)
-        self.assertEqual(result, 0)
+        self.assertEqual(0, result)
 
     def test_user_set_name(self):
         arglist = [
@@ -693,7 +766,6 @@ class TestUserSet(TestUser):
             ('name', 'qwerty'),
             ('password', None),
             ('email', None),
-            ('domain', None),
             ('project', None),
             ('enable', False),
             ('disable', False),
@@ -701,7 +773,6 @@ class TestUserSet(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -726,7 +797,6 @@ class TestUserSet(TestUser):
             ('password', 'secret'),
             ('password_prompt', False),
             ('email', None),
-            ('domain', None),
             ('project', None),
             ('enable', False),
             ('disable', False),
@@ -734,7 +804,6 @@ class TestUserSet(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -759,7 +828,6 @@ class TestUserSet(TestUser):
             ('password', None),
             ('password_prompt', True),
             ('email', None),
-            ('domain', None),
             ('project', None),
             ('enable', False),
             ('disable', False),
@@ -767,7 +835,6 @@ class TestUserSet(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         mocker = mock.Mock()
         mocker.return_value = 'abc123'
         with mock.patch("openstackclient.common.utils.get_password", mocker):
@@ -794,7 +861,6 @@ class TestUserSet(TestUser):
             ('name', None),
             ('password', None),
             ('email', 'barney@example.com'),
-            ('domain', None),
             ('project', None),
             ('enable', False),
             ('disable', False),
@@ -802,7 +868,6 @@ class TestUserSet(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -817,38 +882,6 @@ class TestUserSet(TestUser):
             **kwargs
         )
 
-    def test_user_set_domain(self):
-        arglist = [
-            '--domain', identity_fakes.domain_id,
-            identity_fakes.user_name,
-        ]
-        verifylist = [
-            ('name', None),
-            ('password', None),
-            ('email', None),
-            ('domain', identity_fakes.domain_id),
-            ('project', None),
-            ('enable', False),
-            ('disable', False),
-            ('user', identity_fakes.user_name),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        # DisplayCommandBase.take_action() returns two tuples
-        self.cmd.take_action(parsed_args)
-
-        # Set expected values
-        kwargs = {
-            'enabled': True,
-            'domainId': identity_fakes.domain_id,
-        }
-        # UserManager.update(user, name=, domain=, project=, password=,
-        #     email=, description=, enabled=, default_project=)
-        self.users_mock.update.assert_called_with(
-            identity_fakes.user_id,
-            **kwargs
-        )
-
     def test_user_set_project(self):
         arglist = [
             '--project', identity_fakes.project_id,
@@ -858,7 +891,6 @@ class TestUserSet(TestUser):
             ('name', None),
             ('password', None),
             ('email', None),
-            ('domain', None),
             ('project', identity_fakes.project_id),
             ('enable', False),
             ('disable', False),
@@ -866,13 +898,44 @@ class TestUserSet(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
             'enabled': True,
-            'projectId': identity_fakes.project_id,
+            'default_project': identity_fakes.project_id,
+        }
+        # UserManager.update(user, name=, domain=, project=, password=,
+        #     email=, description=, enabled=, default_project=)
+        self.users_mock.update.assert_called_with(
+            identity_fakes.user_id,
+            **kwargs
+        )
+
+    def test_user_set_project_domain(self):
+        arglist = [
+            '--project', identity_fakes.project_id,
+            '--project-domain', identity_fakes.domain_id,
+            identity_fakes.user_name,
+        ]
+        verifylist = [
+            ('name', None),
+            ('password', None),
+            ('email', None),
+            ('project', identity_fakes.project_id),
+            ('project_domain', identity_fakes.domain_id),
+            ('enable', False),
+            ('disable', False),
+            ('user', identity_fakes.user_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = {
+            'enabled': True,
+            'default_project': identity_fakes.project_id,
         }
         # UserManager.update(user, name=, domain=, project=, password=,
         #     email=, description=, enabled=, default_project=)
@@ -890,7 +953,6 @@ class TestUserSet(TestUser):
             ('name', None),
             ('password', None),
             ('email', None),
-            ('domain', None),
             ('project', None),
             ('enable', True),
             ('disable', False),
@@ -898,7 +960,6 @@ class TestUserSet(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -921,7 +982,6 @@ class TestUserSet(TestUser):
             ('name', None),
             ('password', None),
             ('email', None),
-            ('domain', None),
             ('project', None),
             ('enable', False),
             ('disable', True),
@@ -929,7 +989,6 @@ class TestUserSet(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
         # Set expected values
@@ -944,6 +1003,71 @@ class TestUserSet(TestUser):
         )
 
 
+class TestUserSetPassword(TestUser):
+
+    def setUp(self):
+        super(TestUserSetPassword, self).setUp()
+        self.cmd = user.SetPasswordUser(self.app, None)
+
+    @staticmethod
+    @contextlib.contextmanager
+    def _mock_get_password(*passwords):
+        mocker = mock.Mock(side_effect=passwords)
+        with mock.patch("openstackclient.common.utils.get_password", mocker):
+            yield
+
+    def test_user_password_change(self):
+        current_pass = 'old_pass'
+        new_pass = 'new_pass'
+        arglist = [
+            '--password', new_pass,
+        ]
+        verifylist = [
+            ('password', new_pass),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # Mock getting user current password.
+        with self._mock_get_password(current_pass):
+            self.cmd.take_action(parsed_args)
+
+        self.users_mock.update_password.assert_called_with(
+            current_pass, new_pass
+        )
+
+    def test_user_create_password_prompt(self):
+        current_pass = 'old_pass'
+        new_pass = 'new_pass'
+        parsed_args = self.check_parser(self.cmd, [], [])
+
+        # Mock getting user current and new password.
+        with self._mock_get_password(current_pass, new_pass):
+            self.cmd.take_action(parsed_args)
+
+        self.users_mock.update_password.assert_called_with(
+            current_pass, new_pass
+        )
+
+    def test_user_password_change_no_prompt(self):
+        current_pass = 'old_pass'
+        new_pass = 'new_pass'
+        arglist = [
+            '--password', new_pass,
+            '--original-password', current_pass,
+        ]
+        verifylist = [
+            ('password', new_pass),
+            ('original_password', current_pass),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        self.users_mock.update_password.assert_called_with(
+            current_pass, new_pass
+        )
+
+
 class TestUserShow(TestUser):
 
     def setUp(self):
@@ -967,19 +1091,22 @@ class TestUserShow(TestUser):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
 
         self.users_mock.get.assert_called_with(identity_fakes.user_id)
 
-        collist = ('domain_id', 'email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        collist = ('default_project_id', 'domain_id', 'email',
+                   'enabled', 'id', 'name')
+        self.assertEqual(collist, columns)
         datalist = (
+            identity_fakes.project_id,
             identity_fakes.domain_id,
             identity_fakes.user_email,
             True,
             identity_fakes.user_id,
             identity_fakes.user_name,
-            identity_fakes.project_id,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
