@@ -15,58 +15,54 @@
 
 import logging
 
-from cinderclient.v1 import volume_snapshots
-from cinderclient.v1 import volumes
 from openstackclient.common import utils
 
-# Monkey patch for v1 cinderclient
-volumes.Volume.NAME_ATTR = 'display_name'
-volume_snapshots.Snapshot.NAME_ATTR = 'display_name'
-
 LOG = logging.getLogger(__name__)
 
-DEFAULT_VOLUME_API_VERSION = '1'
+DEFAULT_API_VERSION = '2'
 API_VERSION_OPTION = 'os_volume_api_version'
 API_NAME = "volume"
 API_VERSIONS = {
-    "1": "cinderclient.v1.client.Client"
+    "1": "cinderclient.v1.client.Client",
+    "2": "cinderclient.v2.client.Client"
 }
 
 
 def make_client(instance):
     """Returns a volume service client."""
+
+    # Defer client imports until we actually need them
+    from cinderclient import extension
+    from cinderclient.v1.contrib import list_extensions
+    from cinderclient.v1 import volume_snapshots
+    from cinderclient.v1 import volumes
+
+    # Monkey patch for v1 cinderclient
+    volumes.Volume.NAME_ATTR = 'display_name'
+    volume_snapshots.Snapshot.NAME_ATTR = 'display_name'
+
     volume_client = utils.get_client_class(
         API_NAME,
         instance._api_version[API_NAME],
         API_VERSIONS
     )
-
-    LOG.debug('instantiating volume client')
+    LOG.debug('Instantiating volume client: %s', volume_client)
 
     # Set client http_log_debug to True if verbosity level is high enough
     http_log_debug = utils.get_effective_log_level() <= logging.DEBUG
 
-    client = volume_client(
-        username=instance._username,
-        api_key=instance._password,
-        project_id=instance._project_name,
-        auth_url=instance._auth_url,
-        cacert=instance._cacert,
-        insecure=instance._insecure,
-        region_name=instance._region_name,
-        http_log_debug=http_log_debug
-    )
+    extensions = [extension.Extension('list_extensions', list_extensions)]
 
-    # Populate the Cinder client to skip another auth query to Identity
-    if instance._url:
-        # token flow
-        client.client.management_url = instance._url
-    else:
-        # password flow
-        client.client.management_url = instance.get_endpoint_for_service_type(
-            API_NAME)
-        client.client.service_catalog = instance._service_catalog
-    client.client.auth_token = instance._token
+    # Remember interface only if it is set
+    kwargs = utils.build_kwargs_dict('endpoint_type', instance._interface)
+
+    client = volume_client(
+        session=instance.session,
+        extensions=extensions,
+        http_log_debug=http_log_debug,
+        region_name=instance._region_name,
+        **kwargs
+    )
 
     return client
 
@@ -76,10 +72,8 @@ def build_option_parser(parser):
     parser.add_argument(
         '--os-volume-api-version',
         metavar='<volume-api-version>',
-        default=utils.env(
-            'OS_VOLUME_API_VERSION',
-            default=DEFAULT_VOLUME_API_VERSION),
+        default=utils.env('OS_VOLUME_API_VERSION'),
         help='Volume API version, default=' +
-             DEFAULT_VOLUME_API_VERSION +
+             DEFAULT_API_VERSION +
              ' (Env: OS_VOLUME_API_VERSION)')
     return parser
